/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.configuration;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Date;
import java.util.UUID;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.client.impl.vo.NodeInfo;

/**
 * 
 * @author clemens.duepmeier
 */
public class ConfigurationService {

	private static final Logger logger = LoggerFactory.getLogger(ConfigurationService.class);

	private static final String versionTag;

	private static final String basePath;

	// initialize basePath while loading class
	private static Configuration fileConfig;

	private static Configuration appConfig;

	private static NodeInfo nodeInfo = new NodeInfo();

	private static final String defaultPropertiesFile = System.getProperty("catalina.base") + File.separator + "conf"
			+ File.separator + "soda4LCA.properties";

	static {

		try {
			appConfig = new PropertiesConfiguration("app.properties");
		} catch (ConfigurationException e2) {
			e2.printStackTrace();
		}

		versionTag = appConfig.getString("version.tag");
		logger.info(versionTag);

		URL resourceUrl = Thread.currentThread().getContextClassLoader().getResource("log4j.properties");
		String decodedPath = "";
		// now extract path and decode it
		try {
			// note, that URLs getPath() method does not work, because it don't
			// decode encoded Urls, but URI's does this
			decodedPath = resourceUrl.toURI().getPath();
		} catch (URISyntaxException ex) {
			logger.error("Cannot extract base path from resource files", ex);
		}

		// base path it relative to web application root directory
		basePath = decodedPath.replace("/WEB-INF/classes/log4j.properties", "");
		logger.info("base path of web application: {}", basePath);

		// Obtain our environment naming context
		Context initCtx;
		Context envCtx;
		String propertiesFilePath = null;
		try {
			initCtx = new InitialContext();
			envCtx = (Context) initCtx.lookup("java:comp/env");
			propertiesFilePath = (String) envCtx.lookup("soda4LCAProperties");
			logger.info(propertiesFilePath);
		} catch (NamingException e1) {
			logger.info(e1.getMessage());
		}

		if (propertiesFilePath == null) {
			logger.info("using default application properties at {}", defaultPropertiesFile);
			propertiesFilePath = defaultPropertiesFile;
		} else {
			logger.info("reading application configuration properties from {}", propertiesFilePath);
		}

		try {
			// OK, now load configuration file
			fileConfig = new PropertiesConfiguration(propertiesFilePath);
		} catch (ConfigurationException ex) {
			logger.error(
					"Cannot find application configuration properties file under {}, either put it there or set soda4LCAProperties environment entry via JNDI.",
					propertiesFilePath, ex);
		}
		try {
			nodeInfo.setNodeID(fileConfig.getString("service.node.id"));
		} catch (Exception e) {
			logger.error("Cannot set nodeid from configuration file", e);
		}
		try {
			nodeInfo.setName(fileConfig.getString("service.node.name"));
		} catch (Exception e) {
			logger.error("Cannot set nodename from configuration file", e);
		}

		nodeInfo.setOperator(fileConfig.getString("service.node.operator"));
		nodeInfo.setDescription(fileConfig.getString("service.node.description"));

		nodeInfo.setAdminName(fileConfig.getString("service.admin.name"));
		nodeInfo.setAdminEMail(fileConfig.getString("service.admin.email"));
		nodeInfo.setAdminPhone(fileConfig.getString("service.admin.phone"));
		nodeInfo.setAdminWWW(fileConfig.getString("service.admin.www"));

	}

	public static String getVersionTag() {
		return versionTag;
	}

	public static String getNodeId() {
		return nodeInfo.getNodeID();
	}

	public static String getNodeName() {
		return nodeInfo.getName();
	}

	public static Configuration getProperties() {
		return fileConfig;
	}

	public static NodeInfo getNodeInfo() {
		return nodeInfo;
	}

	public static String getBasePath() {
		return basePath;
	}

	public String getVarDirectory() {
		if (getBasePath().contains("/build/")) {
			return "/";
		} else {
			return getBasePath() + "/WEB-INF/var/";
		}
	}

	public String getZipFileDirectory() {
		return getVarDirectory() + "zips";
	}

	public String getDigitalFileBasePath() {
		return getVarDirectory() + "files";
	}

	public String getUploadDirectory() {
		return getVarDirectory() + "uploads";
	}

	public String getUniqueUploadFileName(String prefix, String extension) {
		StringBuilder buffer = new StringBuilder();
		Date date = new Date();
		buffer.append(prefix).append(date.getTime()).append(UUID.randomUUID()).append(".").append(extension);

		return getUploadDirectory() + "/" + buffer.toString();
	}

}
