/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.dao;

import java.util.HashMap;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.velocity.tools.generic.ValueParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.model.IFlowPropertyListVO;
import de.iai.ilcd.model.flowproperty.FlowProperty;
import de.iai.ilcd.model.unitgroup.UnitGroup;
import de.iai.ilcd.persistence.PersistenceUtil;

/**
 * Data access object for {@link FlowProperty flow properties}
 */
public class FlowPropertyDao extends DataSetDao<FlowProperty, IFlowPropertyListVO> {

	private static final Logger logger = LoggerFactory.getLogger(FlowPropertyDao.class);
	
	/**
	 * Map for the possible incoming keys of order by
	 */
	private final static HashMap<String, String> ORDER_BY_KEY_MAP = FlowPropertyDao.createOrderByKeyMap();
	
	public FlowPropertyDao() {
		super("FlowProperty", FlowProperty.class, IFlowPropertyListVO.class);
	}

	/**
	 * Create the map for the possible incoming keys for order by
	 * @return the map for the possible incoming keys for order by
	 */
	private static HashMap<String, String> createOrderByKeyMap() {
		HashMap<String, String> map = new HashMap<String, String>();
		
		// incoming from lazy data table
		map.put("name.defaultValue","fp.name.defaultValue");
		map.put("classification.classHierarchyAsString","c1.name ascDesc,c2.name");
		map.put("unitGroupName.defaultValue","fp.unitGroup.name.defaultValue");
		map.put("defaultUnit","fp.unitGroup.referenceUnit.name");
		return map;
	}

	@Override
	protected void preCheckAndPersist(FlowProperty dataSet) {
		this.attachUnitGroup(dataSet);
	}

	/**
	 * Get flow property by UUID. Overrides {@link DataSetDao} implementation because
	 * {@link #attachUnitGroup(FlowProperty)} is called after loading.
	 */
	@Override
	public FlowProperty getByUuid(String uuid) {
		FlowProperty fp = super.getByUuid(uuid);
		if (fp != null) {
			this.attachUnitGroup(fp);
		}
		return fp;
	}

	private void attachUnitGroup(FlowProperty flowprop) {
		UnitGroupDao unitGroupDao = new UnitGroupDao();
		if (flowprop != null && flowprop.getReferenceToUnitGroup() != null) {
			String uuid = flowprop.getReferenceToUnitGroup().getUuid().getUuid();
			UnitGroup unitGroup = unitGroupDao.getByUuid(uuid);
			flowprop.setUnitGroup(unitGroup);
		}
	}

	@Override
	protected Query createQueryObject(ValueParser params, String sortString, boolean ascending, boolean returnCount) {

		EntityManager em = PersistenceUtil.getEntityManager();

		boolean joinClassification = !returnCount && "classification.classHierarchyAsString".equals(sortString);
		
		StringBuffer queryString = new StringBuffer();
		if (returnCount) {
			queryString.append("select count(distinct fp) from FlowProperty fp ");
		} else {
			queryString.append("select distinct fp from FlowProperty fp ");
		}

		String orderSql = FlowPropertyDao.ORDER_BY_KEY_MAP.get( sortString );
		if ( orderSql == null ){
			orderSql = "fp.name.defaultValue";
		}
		
		// for order by on classification
		if ( joinClassification ){
			orderSql = orderSql.replaceAll("ascDesc", ascending ? "asc" : "desc");
			queryString.append( this.buildClassificationJoin("fp") );
		}
		
		queryString.append("where ");
		
		// for order by on classification
		if ( joinClassification ){
			queryString.append( this.buildClassificationWhere("fp") );
			queryString.append(" AND ");
		}		
		
		String namePhrase = params.getString("name");
		if (namePhrase == null) {
			namePhrase = "";
		}
		String descriptionPhrase = params.getString("description");
		if (descriptionPhrase == null) {
			descriptionPhrase = "";
		}

		queryString.append("(");

		queryString.append("fp.name.defaultValue like :namePhrase");
		queryString.append(" or fp.synonyms.defaultValue like :namePhrase");
		if (descriptionPhrase.length() > 2) {
			queryString.append(" or fp.description.defaultValue like :descriptionPhrase");
		}
		queryString.append(")");

		// String[] categories = params.getStrings("selectedSectors");
		//
		// if ((categories != null) && (categories.length > 0)) {
		// if (!(categories.length == 1 && categories[0].equals("all"))) {
		// queryString.append(" and ( ");
		// for (int i = 0; i < categories.length; i++) {
		// if (!categories[i].equals("all")) {
		// queryString.append("(dataset.mainCategory='" + categories[i] + "' or dataset.subCategory='" + categories[i] +
		// "')");
		// if (i < categories.length - 1) {
		// queryString.append(" or ");
		// }
		// }
		// }
		// queryString.append(" )");
		// }
		// }

		if (!returnCount){
			queryString.append(" order by "+orderSql+" "+(ascending?"asc":"desc"));
		}
		
		logger.debug("search query: {}", queryString);

		Query query;

		if (descriptionPhrase.length() > 2) {
			query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%")
					.setParameter("descriptionPhrase", "%" + descriptionPhrase + "%");
		} else {
			query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%");
		}

		return query;
	}

	/*
	 * ================================================== == DEPRECATED METHODS / BACKWARDS COMPATIBILITY ==
	 * ==================================================
	 */

	/**
	 * Get list of all contacts
	 * 
	 * @return list of all contacts
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getAll()}.
	 * @see #getAll()
	 */
	public List<FlowProperty> getFlowProperties() {
		return this.getAll();
	}

	/**
	 * Get flow property by UUID
	 * 
	 * @return flow property with provided UUID
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByUuid(String)}.
	 * @see #getByUuid(String)
	 */
	@Deprecated
	public FlowProperty getFlowProperty(String uuid) {
		return this.getByUuid(uuid);
	}

	/**
	 * Get the flow property with the provided JPA id
	 * 
	 * @param datasetId
	 *            JPA id
	 * @return flow property with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(long)}.
	 * @see #getByDataSetId(long)
	 */
	@Deprecated
	public FlowProperty getFlowProperty(long datasetId) {
		return this.getByDataSetId(datasetId);
	}

	/**
	 * Get the flow property with the provided JPA id
	 * 
	 * @param id
	 *            JPA id as string
	 * @return flow property with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(String)}.
	 * @see #getByDataSetId(String)
	 */
	@Deprecated
	public FlowProperty getFlowPropertyById(String id) {
		return this.getByDataSetId(id);
	}

	/**
	 * Get the flow property with the provided JPA id
	 * 
	 * @param id
	 *            JPA id
	 * @return flow property with the provided JPA id
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByDataSetId(long)}.
	 * @see #getByDataSetId(long)
	 */
	@Deprecated
	public FlowProperty getFlowPropertyById(Long id) {
		return this.getByDataSetId(id);
	}

	/**
	 * Get flow properties by main class
	 * 
	 * @param mainClass
	 *            main class to get flow property by
	 * @return matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String)}.
	 * @see #getByClass(String)
	 */
	@Deprecated
	public List<FlowProperty> getFlowPropertiesByClass(String mainClass) {
		return this.getByClass(mainClass);
	}

	/**
	 * Get the number of flow properties by main class
	 * 
	 * @param mainClass
	 *            main class to get flow property by
	 * @return the number of matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String)}.
	 * @see #getNumberByClass(String)
	 */
	@Deprecated
	public long getNumberOfFlowProperties(String mainClass) {
		return this.getNumberByClass(mainClass);
	}

	/**
	 * Get flow properties by main and sub class
	 * 
	 * @param mainClass
	 *            main class to get flow property by
	 * @param subClass
	 *            sub class to get flow property by
	 * @return matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String,String)}.
	 * @see #getByClass(String,String)
	 */
	@Deprecated
	public List<FlowProperty> getFlowPropertiesByClass(String mainClass, String subClass) {
		return this.getByClass(mainClass, subClass);
	}

	/**
	 * Get number of flow properties by main and sub class
	 * 
	 * @param mainClass
	 *            main class to get flow property by
	 * @param subClass
	 *            sub class to get flow property by
	 * @return number of matched data sets
	 * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
	 *             {@link #getByClass(String,String)}.
	 * @see #getNumberByClass(String,String)
	 */
	@Deprecated
	public long getNumberOfFlowProperties(String mainClass, String subClass) {
		return this.getNumberByClass(mainClass, subClass);
	}

}
