/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package de.iai.ilcd.model.dao;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.velocity.tools.generic.ValueParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.model.IFlowPropertyListVO;
import de.fzk.iai.ilcd.service.model.ISourceListVO;
import de.fzk.iai.ilcd.service.model.enums.PublicationTypeValue;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.DigitalFile;
import de.iai.ilcd.model.flowproperty.FlowProperty;
import de.iai.ilcd.model.source.Source;
import de.iai.ilcd.persistence.PersistenceUtil;
import de.schlichtherle.io.File;

/**
 * Data access object for {@link Source sources}
 */
public class SourceDao extends DataSetDao<Source, ISourceListVO> {

    private static final Logger logger = LoggerFactory.getLogger(FlowDao.class);
    /**
     * Map for the possible incoming keys of order by
     */
    private final static HashMap<String, String> ORDER_BY_KEY_MAP = SourceDao.createOrderByKeyMap();

    public SourceDao() {
        super("Source", Source.class, ISourceListVO.class);
    }

    /**
     * Create the map for the possible incoming keys for order by
     * @return the map for the possible incoming keys for order by
     */
    private static HashMap<String, String> createOrderByKeyMap() {
        HashMap<String, String> map = new HashMap<String, String>();

        // incoming from lazy data table
        map.put("name.defaultValue", "s.name.defaultValue");
        map.put("classification.classHierarchyAsString", "c1.name ascDesc,c2.name");
        map.put("publicationType.value", "s.publicationType");
        return map;
    }

    @Override
    protected void preCheckAndPersist(Source dataSet) {
        // nothing to do
    }

    /**
     * Concrete implementation required for saving of digital files
     */
    @Override
    public boolean checkAndPersist(Source dataSet, PersistType pType, PrintWriter out) {
        EntityManager em = PersistenceUtil.getEntityManager();
        EntityTransaction tx = em.getTransaction();

        Source source = (Source) dataSet;

        Source existingSource = this.getByUuid(source.getUuid().getUuid());
        if (existingSource != null) {
            if (pType == PersistType.ONLYNEW) {
                out.println("Warning: source data set with this uuid already exists in database; will ignore this data set");
                return false;
            }
        }
        try {
            tx.begin();
            if (existingSource != null && (pType == PersistType.MERGE)) {
                // delete first the existing one, we will use the new one
                if (out != null) {
                    out.println("Notice: source data set with this uuid already exists in database; will merge this data set");
                }
                em.remove(existingSource);
                deleteDigitalFiles(source);
            }

            em.persist(source);
            tx.commit();
        } catch (Exception e) {
            if (out != null) {
                out.println("Can't save source data file to database because of: " + e.getMessage());
            }
            tx.rollback();
            return false;
        }
        if (source != null && source.getId() != null) {
            if (!saveDigitalFiles(source, out)) {
                if (out != null) {
                    out.println("Warning: couldn't save all files of this source data set into database file directory: see messages above");
                }
            }
        }
        return true;
    }

    private boolean saveDigitalFiles(Source source, PrintWriter out) {
        EntityManager em = PersistenceUtil.getEntityManager();
        EntityTransaction tx = em.getTransaction();

        File directory = null;
        try {

            // OK, now let's handle the files if any
            if ((source.getFiles().size() > 0) && (source.getId() > 0)) { // we have files and the source has a valid id
                // first let's check if the source has arlready a file directory to save binary files
                tx.begin();
                String directoryPath = source.getFilesDirectory();
                directory = new File(directoryPath);

                if (!directory.exists()) {
                    directory.mkdirs(); // OK, create the directory and all parents
                } // OK, now that we verified that we have a directory, let's copy the files to the directory
                for (DigitalFile digitalFile : source.getFiles()) {
                    String sourcePath = digitalFile.getFileName();
                    logger.info("have to save digital file {}", sourcePath);
                    File file = new File(sourcePath);
                    if (file.canRead()) {
                        // copy file only if we have a real file and not only a URL
                        File dest = new File(directory, file.getName());
                        if (!file.copyTo(dest)) {
                            if (out != null) {
                                out.println("cannot copy file " + file.getName() + " of source data set "
                                        + source.getName().getDefaultValue() + " to database file firectory");
                            }
                            logger.error("cannot copy digital file {} to source directory {}", file.getName(),
                                    directoryPath);
                        }
                        // now, replace name in digitalFile with just the name of the file
                        digitalFile.setFileName(FilenameUtils.getName(sourcePath));
                    } else {
                        if (!file.getName().startsWith("http:") || !file.getName().startsWith("https:")) {
                            // there are sometimes URL refs in source which don't have http:// prepended
                            if (!file.getName().matches(".+\\....") && file.getName().contains(".")) {
                                // looks like a URL with no http:// in front; try to fix that
                                digitalFile.setFileName("http://" + file.getName());
                            } else {
                                // we have a file which we cannot find
                                digitalFile.setFileName(FilenameUtils.getName(sourcePath));
                                out.println("warning: digital file " + FilenameUtils.getName(sourcePath)
                                        + " of source data set " + source.getName().getDefaultValue()
                                        + " cannot be found in external_docs directory");
                                logger.info(
                                        "warning: digital file {} of source data set {} cannot be found in external_docs directory; will be ignored",
                                        file.getName(), source.getName().getDefaultValue());
                            }
                        }
                    }
                    em.persist(digitalFile);
                }
                tx.commit();
            }
        } catch (Exception e) {
            // OK, let's delete the digital files and rollback the whole transaction to remove database items
            logger.error("cannot save digital file", e);
            deleteDigitalFiles(source);
            tx.rollback(); // OK, entity also rolled back
            return false;
        }
        return true;
    }

    @Override
    public void remove(DataSet dataSet) throws DeleteDataSetException {
        if (dataSet == null || dataSet.getId() == null) {
            return;
        }
        EntityManager em = PersistenceUtil.getEntityManager();
        EntityTransaction tx = em.getTransaction();
        try {
            tx.begin();
            Source toRemove = em.find(Source.class, dataSet.getId());
            if (toRemove != null) {
                em.remove(toRemove);
                this.deleteDigitalFiles(toRemove);
            }
            tx.commit();
        } catch (Exception e) {
            logger.error("Cannot remove source data set {} from database", dataSet.getName().getDefaultValue(), e);
            if (tx.isActive()) {
                tx.rollback();
                throw new DeleteDataSetException("Cannot remove source data set " + dataSet.getName().getDefaultValue());
            }
        }
    }

    private void deleteDigitalFiles(Source source) {
        File directory = null;
        if (source == null) {
            return;
        }
        if ((source.getFiles().size() > 0) && (source.getId() > 0)) { // we have files and the source has a valid id
            // first let's check if the source has arlready a file directory to save binary files
            String directoryPath = source.getFilesDirectory();
            directory = new File(directoryPath);

            if (directory != null && directory.exists()) {
                try {
                    FileUtils.deleteDirectory(directory);
                } catch (IOException ex) {
                    // we will ignore this because we tried the best to delete everything
                }
            }
        }
        return;
    }

    @Override
    protected Query createQueryObject(ValueParser params, String sortString, boolean ascending, boolean returnCount) {

        EntityManager em = PersistenceUtil.getEntityManager();

        boolean joinClassification = !returnCount && "classification.classHierarchyAsString".equals(sortString);

        StringBuffer queryString = new StringBuffer();
        if (returnCount) {
            queryString.append("select count(distinct s) from Source s ");
        } else {
            queryString.append("select distinct s from Source s ");
        }


        String orderSql = SourceDao.ORDER_BY_KEY_MAP.get(sortString);
        if (orderSql == null) {
            orderSql = "s.name.defaultValue";
        }

        // for order by on classification
        if (joinClassification) {
            orderSql = orderSql.replaceAll("ascDesc", ascending ? "asc" : "desc");
            queryString.append(this.buildClassificationJoin("s"));
        }

        queryString.append("where ");

        // for order by on classification
        if (joinClassification) {
            queryString.append(this.buildClassificationWhere("s"));
            queryString.append(" AND ");
        }



        String namePhrase = params.getString("name");
        if (namePhrase == null) {
            namePhrase = "";
        }
        String descriptionPhrase = params.getString("description");
        if (descriptionPhrase == null) {
            descriptionPhrase = "";
        }

        queryString.append("(");

        queryString.append("s.name.defaultValue like :namePhrase");
        queryString.append(" or s.shortName.defaultValue like :namePhrase");
        if (descriptionPhrase.length() > 2) {
            queryString.append(" or s.description.defaultValue like :descriptionPhrase");
        }
        queryString.append(")");

        String type = params.getString("type");
        if (type != null && (type.length() > 3) && (!type.equals("select option"))) {
            PublicationTypeValue typeValue = null;
            try {
                typeValue = PublicationTypeValue.valueOf(type);
            } catch (Exception e) {
                // ignore it as we do not have a parsable value
            }
            if (typeValue != null) {
                queryString.append(" and ( s.publicationType like '").append(typeValue.value()).append("')");
            }
        }

        // String[] categories = params.getStrings("selectedSectors");
        //
        // if ((categories != null) && (categories.length > 0)) {
        // if (!(categories.length == 1 && categories[0].equals("all"))) {
        // queryString.append(" and ( ");
        // for (int i = 0; i < categories.length; i++) {
        // if (!categories[i].equals("all")) {
        // queryString.append("(dataset.mainCategory='" + categories[i] + "' or dataset.subCategory='" + categories[i] +
        // "')");
        // if (i < categories.length - 1) {
        // queryString.append(" or ");
        // }
        // }
        // }
        // queryString.append(" )");
        // }
        // }


        if (!returnCount) {
            queryString.append(" order by " + orderSql + " " + (ascending ? "asc" : "desc"));
        }
        logger.debug("search query: {}", queryString);

        Query query;

        if (descriptionPhrase.length() > 2) {
            query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%").setParameter("descriptionPhrase", "%" + descriptionPhrase + "%");
        } else {
            query = em.createQuery(queryString.toString()).setParameter("namePhrase", "%" + namePhrase + "%");
        }

        return query;
    }

    /*
     * ================================================== == DEPRECATED METHODS / BACKWARDS COMPATIBILITY ==
     * ==================================================
     */
    /**
     * Get list of all sources
     * 
     * @return list of all sources
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getAll()}.
     * @see #getAll()
     */
    @Deprecated
    public List<Source> getSources() {
        return this.getAll();
    }

    /**
     * Get a source by UUID
     * 
     * @param uuid
     *            UUID of flow to get
     * @return source with provided UUID
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByUuid(String)}.
     * @see #getByUuid(String)
     */
    @Deprecated
    public Source getSource(String uuid) {
        return this.getByUuid(uuid);
    }

    /**
     * Get the source with the provided JPA id
     * 
     * @param datasetId
     *            JPA id
     * @return source with the provided JPA id
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByDataSetId(long)}.
     * @see #getByDataSetId(long)
     */
    @Deprecated
    public Source getSource(long datasetId) {
        return this.getByDataSetId(datasetId);
    }

    /**
     * Get the source with the provided JPA id
     * 
     * @param id
     *            JPA id as string
     * @return source with the provided JPA id
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByDataSetId(String)}.
     * @see #getByDataSetId(String)
     */
    public Source getSourceById(String id) {
        return this.getByDataSetId(id);
    }

    /**
     * Get the source with the provided JPA id
     * 
     * @param id
     *            JPA id
     * @return source with the provided JPA id
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByDataSetId(long)}.
     * @see #getByDataSetId(long)
     */
    public Source getSourceById(Long id) {
        return this.getByDataSetId(id);
    }

    /**
     * Get sources by main class
     * 
     * @param mainClass
     *            main class to get source by
     * @return matched data sets
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByClass(String)}.
     * @see #getByClass(String)
     */
    @Deprecated
    public List<Source> getSourcesByClass(String mainClass) {
        return this.getByClass(mainClass);
    }

    /**
     * Get the number of sources by main class
     * 
     * @param mainClass
     *            main class to get source by
     * @return the number of matched data sets
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByClass(String)}.
     * @see #getNumberByClass(String)
     */
    @Deprecated
    public long getNumberOfSources(String mainClass) {
        return this.getNumberByClass(mainClass);
    }

    /**
     * Get sources by main and sub class
     * 
     * @param mainClass
     *            main class to get source by
     * @param subClass
     *            sub class to get source by
     * @return matched data sets
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByClass(String,String)}.
     * @see #getByClass(String,String)
     */
    @Deprecated
    public List<Source> getSourcesByClass(String mainClass, String subClass) {
        return this.getByClass(mainClass, subClass);
    }

    /**
     * Get number of sources by main and sub class
     * 
     * @param mainClass
     *            main class to get source by
     * @param subClass
     *            sub class to get source by
     * @return number of matched data sets
     * @deprecated still exists for internal backwards compatibility, may be removed in future releases. Use
     *             {@link #getByClass(String,String)}.
     * @see #getNumberByClass(String,String)
     */
    @Deprecated
    public long getNumberOfSources(String mainClass, String subClass) {
        return this.getNumberByClass(mainClass, subClass);
    }
}
