/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.source;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import de.fzk.iai.ilcd.service.model.ISourceVO;
import de.fzk.iai.ilcd.service.model.common.IGlobalReference;
import de.fzk.iai.ilcd.service.model.enums.GlobalReferenceTypeValue;
import de.fzk.iai.ilcd.service.model.enums.PublicationTypeValue;
import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.DigitalFile;
import de.iai.ilcd.model.common.GlobalReference;
import de.iai.ilcd.model.common.MultiLanguageString;
import de.iai.ilcd.model.common.MultiLanguageText;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
@Table(uniqueConstraints = @UniqueConstraint(columnNames = { "UUID", "MAJORVERSION", "MINORVERSION", "SUBMINORVERSION" }))
public class Source extends DataSet implements Serializable, ISourceVO {

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "shortName"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "source_shortname", joinColumns = @JoinColumn(name = "source_id", referencedColumnName = "ID")))
	protected MultiLanguageString shortName = new MultiLanguageString();

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "citation"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "source_citation", joinColumns = @JoinColumn(name = "source_id", referencedColumnName = "ID")))
	protected MultiLanguageText citation = new MultiLanguageText();

	@Enumerated(EnumType.STRING)
	protected PublicationTypeValue publicationType;

	@OneToMany(cascade = CascadeType.ALL, mappedBy = "source")
	Set<DigitalFile> files = new HashSet<DigitalFile>();

	@OneToMany(cascade = CascadeType.ALL)
	Set<GlobalReference> contacts = new HashSet<GlobalReference>();

	@Override
	public MultiLanguageText getCitation() {
		return citation;
	}

	public void setCitation(MultiLanguageText citation) {
		this.citation = citation;
	}

	@Override
	public PublicationTypeValue getPublicationType() {
		return publicationType;
	}

	public void setPublicationType(PublicationTypeValue publicationType) {
		this.publicationType = publicationType;
	}

	public MultiLanguageString getShortName() {
		return shortName;
	}

	public void setShortName(MultiLanguageString shortName) {
		this.shortName = shortName;
	}

	public Set<GlobalReference> getContacts() {
		return contacts;
	}

	/**
	 * Convenience method for returning global references as List in order to user p:dataList (primefaces)
	 * 
	 * @return List of global references
	 */
	public List<GlobalReference> getContactsAsList() {
		return new ArrayList<GlobalReference>(this.getContacts());
	}

	@Override
	public List<IGlobalReference> getBelongsTo() {
		List<IGlobalReference> belongs = new ArrayList<IGlobalReference>();
		if (contacts == null)
			contacts = new HashSet<GlobalReference>();
		for (GlobalReference ref : contacts)
			belongs.add(ref);

		return belongs;
	}

	protected void setContacts(Set<GlobalReference> contacts) {
		this.contacts = contacts;
	}

	public void addContact(GlobalReference contact) {
		if (!contacts.contains(contact))
			contacts.add(contact);
	}

	public Set<DigitalFile> getFiles() {
		return files;
	}

	/**
	 * Convenience method for returning files as List in order to user p:dataList (primefaces)
	 * 
	 * @return List of files
	 */
	public List<DigitalFile> getFilesAsList() {
		return new ArrayList<DigitalFile>(this.getFiles());
	}

	@Override
	public List<IGlobalReference> getFileReferences() {
		List<IGlobalReference> fileRefs = new ArrayList<IGlobalReference>();

		for (DigitalFile file : files) {
			GlobalReference fileRef = new GlobalReference();
			fileRef.setType(GlobalReferenceTypeValue.OTHER_EXTERNAL_FILE);
			fileRef.setShortDescription(new MultiLanguageText(file.getFileName()));
			fileRef.setRefObjectId(this.getUuidAsString());
			// @TODO : provide correct HREF attribute, maybe other too

			// add to list
			fileRefs.add(fileRef);
		}

		return fileRefs;
	}

	protected void setFiles(Set<DigitalFile> files) {
		this.files = files;
	}

	public void addFile(DigitalFile file) {
		if (!files.contains(file)) {
			files.add(file);
			file.setSource(this);
		}
	}

	public String getFilesDirectory() {
		ConfigurationService conf = new ConfigurationService();
		String basePath = conf.getDigitalFileBasePath();
		return basePath + "/" + id.toString();
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.source.Source[id=" + id + "]";
	}

}
