/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.net.UnknownHostException;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.jersey.api.client.ClientHandlerException;

import de.fzk.iai.ilcd.service.client.FailedAuthenticationException;
import de.fzk.iai.ilcd.service.client.FailedConnectionException;
import de.fzk.iai.ilcd.service.client.impl.ILCDNetworkClient;
import de.fzk.iai.ilcd.service.model.INodeInfo;
import de.iai.ilcd.model.dao.NetworkNodeDao;
import de.iai.ilcd.model.dao.PersistType;
import de.iai.ilcd.model.nodes.NetworkNode;

/**
 * 
 * @author clemens.duepmeier
 */

@ManagedBean
@ViewScoped
public class NodeHandler implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3864872300701299028L;

	private static final Logger logger = LoggerFactory.getLogger(NodeHandler.class);

	private NetworkNode node = new NetworkNode();

	private String savedPassword;

	/** Creates a new instance of NodeHandler */
	public NodeHandler() {
		// check if the page was called with a request parameter specifying the current user
		String nodeId = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get("nodeId");
		if (nodeId != null) {
			NetworkNodeDao nodeDao = new NetworkNodeDao();
			NetworkNode existingNode = nodeDao.getNetworkNode(Long.parseLong(nodeId));
			if (existingNode != null) {
				node = existingNode;
				logger.info("found node with node ID {} and database ID {}", node.getNodeId(), node.getId());
				savedPassword = node.getAccessPassword();
			}
		}
	}

	public NetworkNode getNode() {
		return node;
	}

	public void setNode(NetworkNode node) {
		this.node = node;
	}

	public void createNode() {
		NetworkNodeDao dao = new NetworkNodeDao();

		logger.debug("trying to retrieve node info from " + node.getBaseUrl());

		ILCDNetworkClient client = null;

		try {
			// retrieve nodeinfo from foreign node
			client = new ILCDNetworkClient(node.getBaseUrl());
			INodeInfo nodeInfo = client.getNodeInfo();

			logger.debug("retrieved node info for " + nodeInfo.getNodeID());

			// copy properties to internal entity bean
			node.setNodeId(nodeInfo.getNodeID());
			node.setName(nodeInfo.getName());
			node.setOperator(nodeInfo.getOperator());
			node.setBaseUrl(nodeInfo.getBaseURL());
			node.setDescription(nodeInfo.getDescription().getValue());
			node.setAdminName(nodeInfo.getAdminName());
			node.setAdminPhone(nodeInfo.getAdminPhone());
			node.setAdminEmailAddress(nodeInfo.getAdminEMail());
			node.setAdminWwwAddress(nodeInfo.getAdminWWW());

		} catch (FailedConnectionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FailedAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientHandlerException e) {
			if (e.getCause() instanceof UnknownHostException) {
				logger.info("host not found");
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
						"Error: could not connect to host", "");
				FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
				return;
			}
		} catch (Exception e) {
			logger.warn("unknown error retrieving node info");
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "Error: could not retrieve node info",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		NetworkNode existingNode = dao.getNetworkNode(node.getNodeId());
		if (existingNode != null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Error: A network node with this node id already exists", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		if (node.getNodeId() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Error: could not retrieve complete node information from this node, node identifier is missing.",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		if (dao.checkAndPersist(node, PersistType.ONLYNEW, null)) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO,
					"Network node information successfully added to list of network nodes", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} else {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Error: Could not save information to database", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}

	public void changeNode() {
		NetworkNodeDao dao = new NetworkNodeDao();

		if (node.getAccessPassword() == null || node.getAccessPassword().equals("")) {
			// OK, we don't change the password
			node.setAccessPassword(savedPassword);
		}
		NetworkNode existingNode = dao.getNetworkNode(node.getNodeId());
		if (existingNode == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "A node with the node name "
					+ node.getNodeId() + " does not exist", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}
		try {
			logger.info("merge node with node name {} and id {}", node.getNodeId(), node.getId());
			dao.merge(node);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "The node information has been saved",
					"");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} catch (Exception e) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There were some erros while saving the data; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}

}
