/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.ui;

import de.fzk.iai.ilcd.service.model.IDataSetVO;
import java.io.Serializable;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.dao.DataSetDao;

/**
 * Handler for DataSet view
 * 
 * @param <T>
 *            Type of model object that this handler provides
 */
public abstract class AbstractDataSetHandler<I extends IDataSetVO, T extends DataSet, D extends DataSetDao<T, ?>> implements Serializable {

	/**
	 * Serialization ID
	 */
	private static final long serialVersionUID = -3151934201431439509L;

	/**
	 * Instance to be hold
	 */
	private I dataSet;

	/**
	 * DAO for data retrieval
	 */
	private final D daoInstance;

	/**
	 * {@link DataSet#getId()Data set id} as string
	 */
	private String dataSetIdString = null;

	/**
	 * Initialize handler
	 * 
	 * @param daoInstance
	 *            matching DAO
	 */
	public AbstractDataSetHandler(D daoInstance) {
		super();
		this.daoInstance = daoInstance;
	}

	/**
	 * Set the data set
	 * 
	 * @param dataSet
	 *            data set to set
	 */
	public void setDataSet(I dataSet) {
		this.dataSet = dataSet;
	}

	/**
	 * Get the data set
	 * 
	 * @return data set
	 */
	public I getDataSet() {
		return this.dataSet;
	}

	/**
	 * Get the data set id as string
	 * 
	 * @return data set id as string
	 */
	public String getDataSetIdString() {
		return this.dataSetIdString;
	}

	/**
	 * Set the data set ID as string, will trigger loading the object via provided DAO
	 * 
	 * @param dataSetIdString
	 *            the data set ID as string
	 */
	public void setDataSetIdString(String dataSetIdString) {
		this.setDataSetIdString(dataSetIdString, true);
	}

	/**
	 * Set the data set ID as string
	 * 
	 * @param dataSetIdString
	 *            the data set ID as string
	 * @param doLoad
	 *            indicate if data should be loaded from dao
	 */
	protected void setDataSetIdString(String dataSetIdString, boolean doLoad) {
		this.dataSetIdString = dataSetIdString;
		if (doLoad) {
			if (this.dataSetIdString != null) {
				this.dataSet = (I)this.daoInstance.getByDataSetId(this.dataSetIdString);
			} else {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
						"There's no id, therefore no data could be found", "");
				FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			}
		}
	}

	/**
	 * Get the DAO instance of this handler
	 * 
	 * @return DAO instance of this handler
	 */
	protected final D getDaoInstance() {
		return this.daoInstance;
	}

	/**
	 * Determine if string is <code>null</code> or {@link String#indexOf(int) empty}. Please note that if string is not
	 * <code>null</code>, it is {@link String#trim() trimmed} before empty check.
	 * 
	 * @param s
	 *            string to test
	 * @return <code>true</code> if <code>null</code> or empty, else <code>false</code>
	 */
	protected final boolean isStringNullOrEmpty(String s) {
		return s == null || s.trim().isEmpty();
	}

}
