/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.webgui.controller.ui;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import javax.faces.bean.ManagedBean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.api.Constants;
import de.fzk.iai.ilcd.zip.ILCDManifest;
import de.fzk.iai.ilcd.zip.ILCDZip;
import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.DigitalFile;
import de.iai.ilcd.model.dao.ContactDao;
import de.iai.ilcd.model.dao.DataSetDao;
import de.iai.ilcd.model.dao.FlowDao;
import de.iai.ilcd.model.dao.FlowPropertyDao;
import de.iai.ilcd.model.dao.LCIAMethodDao;
import de.iai.ilcd.model.dao.ProcessDao;
import de.iai.ilcd.model.dao.SourceDao;
import de.iai.ilcd.model.dao.UnitGroupDao;
import de.iai.ilcd.model.source.Source;
import de.iai.ilcd.webgui.controller.admin.DataExportResource;
import de.schlichtherle.io.ArchiveException;
import de.schlichtherle.io.File;
import de.schlichtherle.io.FileWriter;

/**
 * Backing bean for data export
 */
@ManagedBean
public class DataExportHandler {

	private final Logger logger = LoggerFactory.getLogger(DataExportHandler.class);

	/**
	 * Initialize handler
	 */
	public DataExportHandler() {
	}

	public String getExportResourcePath() {
		DataExportResource r = new DataExportResource();
		return r.getRequestPath();
	}

	public ILCDZip export() throws IOException {

		logger.info("starting export of entire database.");

		ConfigurationService confService = new ConfigurationService();

		// the new archive
		ILCDZip zip = new ILCDZip(confService.getZipFileDirectory() + File.separator
				+ Long.toString(System.currentTimeMillis()) + ".zip");

		ILCDManifest manifest = new ILCDManifest(Constants.FORMAT_VERSION, ConfigurationService.getVersionTag());
		zip.setManifest(manifest);

		long start = System.currentTimeMillis();

		logger.info("exporting processes");
		exportDatasets(new ProcessDao(), zip);
		logger.info("exporting LCIA methods");
		exportDatasets(new LCIAMethodDao(), zip);
		logger.info("exporting sources");
		exportDatasets(new SourceDao(), zip);
		logger.info("exporting contacts");
		exportDatasets(new ContactDao(), zip);
		logger.info("exporting flows");
		exportDatasets(new FlowDao(), zip);
		logger.info("exporting flow props");
		exportDatasets(new FlowPropertyDao(), zip);
		logger.info("exporting unit groups");
		exportDatasets(new UnitGroupDao(), zip);

		try {
			ILCDZip.umount(zip);
		} catch (ArchiveException e) {
			logger.error("error writing archive", e);
			return null;
		}
		long stop = System.currentTimeMillis();

		logger.info("done. export took " + (stop - start) / 1000 + " ms");

		return zip;
	}

	private <T extends DataSet> void exportDatasets(DataSetDao<T, ?> dao, ILCDZip zip) {
		List<T> dataSetList = dao.getAll();

		File dir = null;

		if (dao instanceof ProcessDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "processes");
		} else if (dao instanceof LCIAMethodDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "lciamethods");
		} else if (dao instanceof FlowDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "flows");
		} else if (dao instanceof FlowPropertyDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "flowproperties");
		} else if (dao instanceof UnitGroupDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "unitgroups");
		} else if (dao instanceof SourceDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "sources");
			File externalDocsDir = new File(zip.getAbsolutePath() + File.separator + "external_docs");
			externalDocsDir.mkdir();
		} else if (dao instanceof ContactDao) {
			dir = new File(zip.getAbsolutePath() + File.separator + "contacts");
		}

		if (dir != null)
			dir.mkdir();
		else
			throw new IllegalArgumentException("Invalid DAO object");

		if (dataSetList.equals(Collections.EMPTY_LIST))
			return;

		BufferedWriter writer = null;

		for (T dataset : dataSetList) {

			try {
				long id = dataset.getId();

				T dataset2 = dao.getByDataSetId(id);

				writer = new BufferedWriter(new FileWriter(dir.getAbsolutePath() + File.separator
						+ dataset2.getUuidAsString() + ".xml"));
				writer.write(dataset2.getXmlFile().getContent());
				writer.close();

				if (dataset2 instanceof Source) {
					for (DigitalFile file : ((Source) dataset2).getFiles()) {
						File digitalFile = new File(file.getAbsoluteFileName());
						if (digitalFile.exists())
							ILCDZip.cp(digitalFile, new File(zip.getAbsolutePath() + File.separator + "external_docs"
									+ File.separator + file.getFileName()));
					}
				}
			} catch (Exception e) {
				logger.error("error writing to archive", e);
			} finally {
				try {
					if (writer != null)
						writer.close();
				} catch (Exception e) {
					logger.error("error writing to archive", e);
				}
			}
		}
	}
}
