/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package de.iai.ilcd.webgui.controller.ui;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ComponentSystemEvent;

import de.fzk.iai.ilcd.service.model.IProcessVO;
import de.fzk.iai.ilcd.service.model.enums.LCIMethodApproachesValue;
import de.fzk.iai.ilcd.service.model.process.IComplianceSystem;
import de.iai.ilcd.model.dao.ProcessDao;
import de.iai.ilcd.model.process.Process;
import java.util.ArrayList;
import java.util.List;

/**
 * Backing bean for process detail view
 */
@ManagedBean
public class ProcessHandler extends AbstractDataSetHandler<IProcessVO, Process, ProcessDao> {

    /**
     * Serialization ID
     */
    private static final long serialVersionUID = 6907352892423743765L;
    /**
     * Source node foreign process
     */
    private String sourceNode = null;
    /**
     * UUID of foreign process
     */
    private String uuid = null;

    /**
     * Initialize the handler
     */
    public ProcessHandler() {
        super(new ProcessDao());
    }

    /**
     * <p>
     * Set the data set ID as string.
     * </p>
     * <p>
     * Will <strong>not</strong> trigger loading the object via provided DAO
     * </p>
     * 
     * @param dataSetIdString
     *            the data set ID as string
     */
    @Override
    public void setDataSetIdString(String dataSetIdString) {
        super.setDataSetIdString(dataSetIdString, false);
    }

    /**
     * Convenience method, delegates to {@link #getDataSet()}
     * 
     * @return process
     */
    public IProcessVO getProcess() {
        return this.getDataSet();
    }

    /**
     * Trigger loading of process.
     * 
     * @return process
     */
    public void postViewParamInit(ComponentSystemEvent event) throws AbortProcessingException {
        IProcessVO p = null;
        // id is defined
        if (!this.isStringNullOrEmpty(this.getDataSetIdString())) {
            // try to load by string id, if string id is null or empty, null will be returned
            p = this.getDaoInstance().getByDataSetId(this.getDataSetIdString());
        } // id not defined => foreign
        else if (!this.isStringNullOrEmpty(this.sourceNode) && !this.isStringNullOrEmpty(this.uuid)) {
            IProcessVO ip = this.getDaoInstance().getForeignProcess(this.sourceNode, this.uuid);
            // foreign process found
            if (ip != null) {
                //p = new Process(ip);
                p = ip;
            } // foreign not found
            else {
                FacesMessage facesMsg = new FacesMessage(
                        FacesMessage.SEVERITY_WARN,
                        "Although there's a sourceNode and also an uuid, no IProcessVO (and therefore no process) could be found",
                        "");
                FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
            }
        } else {
            FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
                    "There's neither id nor sourceNode and uuid, therefore no process could be found", "");
            FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
        }

        this.setDataSet(p);
    }

    /**
     * Get the source node as string
     * 
     * @return source node as string
     */
    public String getSourceNode() {
        return this.sourceNode;
    }

    /**
     * Set the source node
     * 
     * @param sourceNode
     *            source node to set
     */
    public void setSourceNode(String sourceNode) {
        this.sourceNode = sourceNode;
    }

    /**
     * Get the UUID as string
     * 
     * @return UUID as string
     */
    public String getUuid() {
        return this.uuid;
    }

    /**
     * Set the UUID as string
     * 
     * @param uuid
     *            UUID as string to set
     */
    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    /**
     * We need this as workaround because the Service API delivers a set but we need a List in the template
     * @return 
     */
    public List<LCIMethodApproachesValue> getApproaches() {
        List<LCIMethodApproachesValue> approaches = new ArrayList<LCIMethodApproachesValue>();

        if (this.getDataSet() != null) {
            approaches = new ArrayList<LCIMethodApproachesValue>(this.getDataSet().getLCIMethodInformation().getApproaches());
        }

        return approaches;
    }
    
    
    /**
     * We need this too as workaround to geth the ComplianceSystems as List
     * @return 
     */
    public List<IComplianceSystem> getComplianceSystems() {
        List<IComplianceSystem> complianceSystems = new ArrayList<IComplianceSystem>();

        if (this.getDataSet() != null) {
            complianceSystems = new ArrayList<IComplianceSystem>(this.getDataSet().getComplianceSystems());
        }

        return complianceSystems;
    }
}
